@@ -2,11 +2,11 @@
  * @fileoverview Magic dependency loading. Taken from script.aculo.us
  * and modified to break it.
  * @author Stefan Strigler steve@zeank.in-berlin.de 
- * @version $Revision: 456 $
+ * @version $Revision$
  */
 
 var JSJaC = {
-  Version: '$Rev: 456 $',
+  Version: '$Rev$',
   require: function(libraryName) {
     // inserting via DOM fails in Safari 2.0, so brute force approach
     document.write('<script type="text/javascript" src="'+libraryName+'"></script>');
@@ -41,12 +41,9 @@ var JSJaC = {
     for (var i=0; i<includes.length; i++)
       this.require(path+includes[i]+'.js');
   },
-  bind: function(fn, obj, arg) {
-    return function() {
-      if (arg)
-        fn.apply(obj, arg);
-      else
-        fn.apply(obj);
+  bind: function(fn, obj, optArg) {
+    return function(arg) {
+      return fn.apply(obj, [arg, optArg]);
     };
   }
 };
