@@ -2,7 +2,7 @@
  * @fileoverview Contains all things in common for all subtypes of connections
  * supported.
  * @author Stefan Strigler steve@zeank.in-berlin.de
- * @version $Revision: 476 $
+ * @version $Revision$
  */
 
 /**
@@ -16,32 +16,45 @@
  * * <code>oDbg</code> (optional) a reference to a debugger interface
  */
 function JSJaCConnection(oArg) {
-
-  if (oArg && oArg.oDbg && oArg.oDbg.log)
-    /**
-     * Reference to debugger interface
-     *(needs to implement method <code>log</code>)
-     * @type Debugger
-     */
+  
+  if (oArg && oArg.oDbg && oArg.oDbg.log) {
+      /**
+       * Reference to debugger interface
+       * (needs to implement method <code>log</code>)
+       * @type Debugger
+       */
     this.oDbg = oArg.oDbg;
-  else {
+  } else {
     this.oDbg = new Object(); // always initialise a debugger
     this.oDbg.log = function() { };
   }
+  
+  if (oArg && oArg.timerval)
+    this.setPollInterval(oArg.timerval);
+  else 
+    this.setPollInterval(JSJAC_TIMERVAL);
 
   if (oArg && oArg.httpbase)
-    /**
-     * @private
-     */
+      /**
+       * @private
+       */
     this._httpbase = oArg.httpbase;
  
-  if (oArg && oArg.allow_plain)
-    /**
-     * @private
-     */
+  if (oArg &&oArg.allow_plain)
+      /**
+       * @private
+       */
     this.allow_plain = oArg.allow_plain;
   else
     this.allow_plain = JSJAC_ALLOW_PLAIN;
+  
+  if (oArg && oArg.cookie_prefix)
+      /**
+       * @private
+       */
+    this._cookie_prefix = oArg.cookie_prefix;
+  else
+    this._cookie_prefix = "";
 
   /**
    * @private
@@ -91,9 +104,6 @@ function JSJaCConnection(oArg) {
    * @private
    */
   this._sendRawCallbacks = new Array();
-
-  if (oArg && oArg.timerval)
-    this.setPollInterval(oArg.timerval);
 }
 
 JSJaCConnection.prototype.connect = function(oArg) {
@@ -187,7 +197,7 @@ JSJaCConnection.prototype.disconnect = function() {
   this._req[slot].r.send(request);
 
   try {
-    JSJaCCookie.read('JSJaC_State').erase();
+    JSJaCCookie.read(this._cookie_prefix+'JSJaC_State').erase();
   } catch (e) {}
 
   this.oDbg.log("Disconnected: "+this._req[slot].r.responseText,2);
@@ -364,16 +374,28 @@ JSJaCConnection.prototype.registerIQSet =
  */
 JSJaCConnection.prototype.resume = function() {
   try {
-    this._setStatus('resuming');
-    var s = unescape(JSJaCCookie.read('JSJaC_State').getValue());
-     
-    this.oDbg.log('read cookie: '+s,2);
+    var json = JSJaCCookie.read(this._cookie_prefix+'JSJaC_State').getValue(); 
+    this.oDbg.log('read cookie: '+json,2);
+    JSJaCCookie.read(this._cookie_prefix+'JSJaC_State').erase();
 
-    var o = JSJaCJSON.parse(s);
-     
-    for (var i in o)
-      if (o.hasOwnProperty(i))
-        this[i] = o[i];
+    return this.resumeFromData(JSJaCJSON.parse(json));
+  } catch (e) {}
+  return false; // sth went wrong
+};
+
+/**
+ * Resumes BOSH connection from data  
+ * @param {Object} serialized jsjac state information
+ * @return Whether resume was successful
+ * @type boolean
+ */
+JSJaCConnection.prototype.resumeFromData = function(data) {
+  try {
+    this._setStatus('resuming');
+
+    for (var i in data)
+      if (data.hasOwnProperty(i))
+        this[i] = data[i];
      
     // copy keys - not being very generic here :-/
     if (this._keys) {
@@ -384,10 +406,6 @@ JSJaCConnection.prototype.resume = function() {
       this._keys = this._keys2;
     }
 
-    try {
-      JSJaCCookie.read('JSJaC_State').erase();
-    } catch (e) {}
-
     if (this._connected) {
       // don't poll too fast!
       this._handleEvent('onresume');
@@ -465,17 +483,13 @@ JSJaCConnection.prototype.sendIQ = function(iq, handlers, arg) {
   }
 
   handlers = handlers || {};
-  var error_handler = handlers.error_handler || function(aIq) {
-    this.oDbg.log(iq.xml(), 1);
-  };
+    var error_handler = handlers.error_handler || JSJaC.bind(function(aIq) {
+        this.oDbg.log(aIq.xml(), 1);
+    }, this);
  
-  var result_handler = handlers.result_handler ||  function(aIq) {
-    this.oDbg.log(aIq.xml(), 2);
-  };
-  // unsure, what's the use of this?
-  var default_handler = handlers.default_handler || function(aIq) {
-    this.oDbg.log(aIq.xml(), 2);
-  };
+    var result_handler = handlers.result_handler ||  JSJaC.bind(function(aIq) {
+        this.oDbg.log(aIq.xml(), 2);
+    }, this);
 
   var iqHandler = function(aIq, arg) {
     switch (aIq.getType()) {
@@ -485,8 +499,6 @@ JSJaCConnection.prototype.sendIQ = function(iq, handlers, arg) {
       case 'result':
       result_handler(aIq, arg);
       break;
-      default: // may it be?
-      default_handler(aIq, arg);
     }
   };
   return this.send(iq, iqHandler, arg);
@@ -524,55 +536,68 @@ JSJaCConnection.prototype.setPollInterval = function(timerval) {
 JSJaCConnection.prototype.status = function() { return this._status; };
 
 /**
- * Suspsends this connection (saving state for later resume)
+ * Suspends this connection (saving state for later resume)
+ * Saves state to cookie
+ * @return Whether suspend (saving to cookie) was successful
+ * @type boolean
  */
 JSJaCConnection.prototype.suspend = function() {
-	
-    // remove timers
-    clearTimeout(this._timeout);
-    clearInterval(this._interval);
-    clearInterval(this._inQto);
-
-    this._suspend();
-
-    var u = ('_connected,_keys,_ID,_inQ,_pQueue,_regIDs,_errcnt,_inactivity,domain,username,resource,jid,fulljid,_sid,_httpbase,_timerval,_is_polling').split(',');
-    u = u.concat(this._getSuspendVars());
-    var s = new Object();
-
-    for (var i=0; i<u.length; i++) {
-      if (!this[u[i]]) continue; // hu? skip these!
-      if (this[u[i]]._getSuspendVars) {
-        var uo = this[u[i]]._getSuspendVars();
-        var o = new Object();
-        for (var j=0; j<uo.length; j++)
-          o[uo[j]] = this[u[i]][uo[j]];
-      } else
-        var o = this[u[i]];
-
-      s[u[i]] = o;
-    }
-    var c = new JSJaCCookie('JSJaC_State', escape(JSJaCJSON.toString(s)),
-                            this._inactivity);
-    this.oDbg.log("writing cookie: "+unescape(c.value)+"\n(length:"+
-                  unescape(c.value).length+")",2);
+  var data = this.suspendToData();
+  
+  try {
+    var c = new JSJaCCookie(this._cookie_prefix+'JSJaC_State', JSJaCJSON.toString(data));
+    this.oDbg.log("writing cookie: "+c.getValue()+"\n"+
+                  "(length:"+c.getValue().length+")",2);
     c.write();
 
-    try {
-      var c2 = JSJaCCookie.read('JSJaC_State');
-      if (c.value != c2.value) {
-        this.oDbg.log("Suspend failed writing cookie.\nRead: "+
-                      unescape(JSJaCCookie.read('JSJaC_State')), 1);
-        c.erase();
-      }
-
-      this._connected = false;
-
-      this._setStatus('suspending');
-    } catch (e) {
-      this.oDbg.log("Failed reading cookie 'JSJaC_State': "+e.message);
+    var c2 = JSJaCCookie.get(this._cookie_prefix+'JSJaC_State');
+    if (c.getValue() != c2) {
+      this.oDbg.log("Suspend failed writing cookie.\nread: " + c2, 1);
+      c.erase();
+      return false;
     }
+    return true;
+  } catch (e) {
+    this.oDbg.log("Failed creating cookie '"+this._cookie_prefix+
+                  "JSJaC_State': "+e.message,1);
+  }
+  return false;
+};
 
-  };
+/**
+ * Suspend connection and return serialized JSJaC connection state
+ * @return JSJaC connection state object
+ * @type Object
+ */
+JSJaCConnection.prototype.suspendToData = function() {
+  
+  // remove timers
+  clearTimeout(this._timeout);
+  clearInterval(this._interval);
+  clearInterval(this._inQto);
+
+  this._suspend();
+
+  var u = ('_connected,_keys,_ID,_inQ,_pQueue,_regIDs,_errcnt,_inactivity,domain,username,resource,jid,fulljid,_sid,_httpbase,_timerval,_is_polling').split(',');
+  u = u.concat(this._getSuspendVars());
+  var s = new Object();
+
+  for (var i=0; i<u.length; i++) {
+    if (!this[u[i]]) continue; // hu? skip these!
+    if (this[u[i]]._getSuspendVars) {
+      var uo = this[u[i]]._getSuspendVars();
+      var o = new Object();
+      for (var j=0; j<uo.length; j++)
+        o[uo[j]] = this[u[i]][uo[j]];
+    } else
+      var o = this[u[i]];
+
+    s[u[i]] = o;
+  }
+  this._connected = false;
+  this._setStatus('suspending');
+  return s;
+};
 
 /**
  * @private
@@ -824,7 +849,7 @@ JSJaCConnection.prototype._doSASLAuthDigestMd5S1 = function(el) {
     var rPlain = 'username="'+this.username+'",realm="'+this.domain+
     '",nonce="'+this._nonce+'",cnonce="'+this._cnonce+'",nc="'+this._nc+
     '",qop=auth,digest-uri="'+this._digest_uri+'",response="'+response+
-    '",charset=utf-8';
+    '",charset="utf-8"';
    
     this.oDbg.log("response: "+rPlain,2);
 
@@ -893,7 +918,7 @@ JSJaCConnection.prototype._doSASLAuthDone = function (el) {
  */
 JSJaCConnection.prototype._doStreamBind = function() {
   var iq = new JSJaCIQ();
-  iq.setIQ(this.domain,'set','bind_1');
+  iq.setIQ(null,'set','bind_1');
   iq.appendNode("bind", {xmlns: "urn:ietf:params:xml:ns:xmpp-bind"},
                 [["resource", this.resource]]);
   this.oDbg.log(iq.xml());
@@ -946,7 +971,7 @@ JSJaCConnection.prototype._handleEvent = function(event,arg) {
   this.oDbg.log("handling event '"+event+"'",2);
   for (var i=0;i<this._events[event].length; i++) {
     var aEvent = this._events[event][i];
-    if (aEvent.handler) {
+    if (typeof aEvent.handler == 'function') {
       try {
         if (arg) {
           if (arg.pType) { // it's a packet
@@ -959,13 +984,25 @@ JSJaCConnection.prototype._handleEvent = function(event,arg) {
               continue;
             this.oDbg.log(aEvent.childName+"/"+aEvent.childNS+"/"+aEvent.type+" => match for handler "+aEvent.handler,3);
           }
-          if (aEvent.handler.call(this,arg)) // handled!
+          if (aEvent.handler(arg)) {
+            // handled!
             break;
+          }
         }
         else
-          if (aEvent.handler.call(this)) // handled!
+          if (aEvent.handler()) {
+            // handled!
             break;
-      } catch (e) { this.oDbg.log(aEvent.handler+"\n>>>"+e.name+": "+ e.message,1); }
+          }
+      } catch (e) { 
+
+        if (e.fileName&&e.lineNumber) {
+            this.oDbg.log(aEvent.handler+"\n>>>"+e.name+": "+ e.message+' in '+e.fileName+' line '+e.lineNumber,1);    
+        } else {
+            this.oDbg.log(aEvent.handler+"\n>>>"+e.name+": "+ e.message,1);             
+        }
+
+      }
     }
   }
 };
@@ -982,7 +1019,7 @@ JSJaCConnection.prototype._handlePID = function(aJSJaCPacket) {
       var pID = aJSJaCPacket.getID();
       this.oDbg.log("handling "+pID,3);
       try {
-        if (this._regIDs[i].cb.call(this, aJSJaCPacket,this._regIDs[i].arg) === false) {
+        if (this._regIDs[i].cb.call(this, aJSJaCPacket, this._regIDs[i].arg) === false) {
           // don't unregister
           return false;
         } else {
@@ -991,7 +1028,7 @@ JSJaCConnection.prototype._handlePID = function(aJSJaCPacket) {
         }
       } catch (e) {
         // broken handler?
-        this.oDbg.log(e.name+": "+ e.message);
+        this.oDbg.log(e.name+": "+ e.message, 1);
         this._unregisterPID(pID);
         return true;
       }
@@ -1068,8 +1105,8 @@ JSJaCConnection.prototype._parseStreamFeatures = function(doc) {
   if (this.has_sasl)
     this.oDbg.log("SASL detected",2);
   else {
-    this.authtype = 'nonsasl';
     this.oDbg.log("No support for SASL detected",2);
+    return false;
   }
 
   /* [TODO]
