@@ -1,7 +1,7 @@
 /**
  * @fileoverview Contains all Jabber/XMPP packet related classes.
  * @author Stefan Strigler steve@zeank.in-berlin.de
- * @version $Revision: 480 $
+ * @version $Revision$
  */
 
 var JSJACPACKET_USE_XMLNS = true;
@@ -176,7 +176,7 @@ JSJaCPacket.prototype.getXMLLang = function() {
  * @type String
  */
 JSJaCPacket.prototype.getXMLNS = function() {
-  return this.getNode().namespaceURI;
+  return this.getNode().namespaceURI || this.getNode().getAttribute('xmlns');
 };
 
 /**
@@ -190,7 +190,7 @@ JSJaCPacket.prototype.getChild = function(name, ns) {
   if (!this.getNode()) {
     return null;
   }
- 
+
   name = name || '*';
   ns = ns || '*';
 
@@ -202,7 +202,7 @@ JSJaCPacket.prototype.getChild = function(name, ns) {
   var nodes = this.getNode().getElementsByTagName(name);
   if (ns != '*') {
     for (var i=0; i<nodes.length; i++) {
-      if (nodes.item(i).namespaceURI == ns) {
+      if (nodes.item(i).namespaceURI == ns || nodes.item(i).getAttribute('xmlns') == ns) {
         return nodes.item(i);
       }
     }
@@ -210,7 +210,7 @@ JSJaCPacket.prototype.getChild = function(name, ns) {
     return nodes.item(0);
   }
   return null; // nothing found
-}
+};
 
 /**
  * Gets the node value of a child element of this packet.
@@ -294,27 +294,66 @@ JSJaCPacket.prototype._getAttribute = function(attr) {
   return this.getNode().getAttribute(attr);
 };
 
+
+if (document.ELEMENT_NODE == null) {
+  document.ELEMENT_NODE = 1;
+  document.ATTRIBUTE_NODE = 2;
+  document.TEXT_NODE = 3;
+  document.CDATA_SECTION_NODE = 4;
+  document.ENTITY_REFERENCE_NODE = 5;
+  document.ENTITY_NODE = 6;
+  document.PROCESSING_INSTRUCTION_NODE = 7;
+  document.COMMENT_NODE = 8;
+  document.DOCUMENT_NODE = 9;
+  document.DOCUMENT_TYPE_NODE = 10;
+  document.DOCUMENT_FRAGMENT_NODE = 11;
+  document.NOTATION_NODE = 12;
+}
+
 /**
- * Replaces this node with given node
+ * import node into this packets document
  * @private
  */
-JSJaCPacket.prototype._replaceNode = function(aNode) {
-  // copy attribs
-  for (var i=0; i<aNode.attributes.length; i++)
-    if (aNode.attributes.item(i).nodeName != 'xmlns')
-      this.getNode().setAttribute(aNode.attributes.item(i).nodeName,
-                                  aNode.attributes.item(i).nodeValue);
+JSJaCPacket.prototype._importNode = function(node, allChildren) {
+  switch (node.nodeType) {
+  case document.ELEMENT_NODE:
 
-  // copy children
-  for (var i=0; i<aNode.childNodes.length; i++)
-    if (this.getDoc().importNode)
-      this.getNode().appendChild(this.getDoc().importNode(aNode.
-                                                          childNodes.item(i),
-                                                          true));
-    else
-      this.getNode().appendChild(aNode.childNodes.item(i).cloneNode(true));
+  if (this.getDoc().createElementNS) {
+    var newNode = this.getDoc().createElementNS(node.namespaceURI, node.nodeName);
+  } else {
+    var newNode = this.getDoc().createElement(node.nodeName);
+  }
+
+  /* does the node have any attributes to add? */
+  if (node.attributes && node.attributes.length > 0)
+    for (var i = 0, il = node.attributes.length;i < il; i++) {
+      var attr = node.attributes.item(i);
+      if (attr.nodeName == 'xmlns' && newNode.getAttribute('xmlns') != null ) continue;
+      if (newNode.setAttributeNS && attr.namespaceURI) {
+        newNode.setAttributeNS(attr.namespaceURI,
+                               attr.nodeName,
+                               attr.nodeValue);
+      } else {
+        newNode.setAttribute(attr.nodeName,
+                             attr.nodeValue);
+      }
+    }
+  /* are we going after children too, and does the node have any? */
+  if (allChildren && node.childNodes && node.childNodes.length > 0) {
+    for (var i = 0, il = node.childNodes.length; i < il; i++) {
+      newNode.appendChild(this._importNode(node.childNodes.item(i), allChildren));
+    }
+  }
+  return newNode;
+  break;
+  case document.TEXT_NODE:
+  case document.CDATA_SECTION_NODE:
+  case document.COMMENT_NODE:
+  return this.getDoc().createTextNode(node.nodeValue);
+  break;
+  }
 };
- 
+
 /**
  * Set node value of a child node
  * @private
@@ -523,11 +562,9 @@ JSJaCIQ.prototype.setQuery = function(xmlns) {
   try {
     query = this.getDoc().createElementNS(xmlns,'query');
   } catch (e) {
-    // fallback
     query = this.getDoc().createElement('query');
+	query.setAttribute('xmlns',xmlns);
   }
-  if (query && query.getAttribute('xmlns') != xmlns) // fix opera 8.5x
-    query.setAttribute('xmlns',xmlns);
   this.getNode().appendChild(query);
   return query;
 };
@@ -546,10 +583,11 @@ JSJaCIQ.prototype.getQuery = function() {
  * @type String
  */
 JSJaCIQ.prototype.getQueryXMLNS = function() {
-  if (this.getQuery())
-    return this.getQuery().namespaceURI;
-  else
+  if (this.getQuery()) {
+    return this.getQuery().namespaceURI || this.getQuery().getAttribute('xmlns');
+  } else {
     return null;
+  }
 };
 
 /**
@@ -561,6 +599,7 @@ JSJaCIQ.prototype.getQueryXMLNS = function() {
 JSJaCIQ.prototype.reply = function(payload) {
   var rIQ = this.clone();
   rIQ.setTo(this.getFrom());
+  rIQ.setFrom();
   rIQ.setType('result');
   if (payload) {
     if (typeof payload == 'string')
@@ -662,23 +701,25 @@ JSJaCMessage.prototype.getSubject = function() {
  * @type JSJaCPacket
  */
 JSJaCPacket.wrapNode = function(node) {
-  var aNode;
+  var oPacket = null;
+
   switch (node.nodeName.toLowerCase()) {
   case 'presence':
-    aNode = new JSJaCPresence();
-    break;
+      oPacket = new JSJaCPresence();
+      break;
   case 'message':
-    aNode = new JSJaCMessage();
-    break;
+      oPacket = new JSJaCMessage();
+      break;
   case 'iq':
-    aNode = new JSJaCIQ();
-    break;
-  default : // unknown
-    return null;
+      oPacket = new JSJaCIQ();
+      break;
   }
 
-  aNode._replaceNode(node);
+  if (oPacket) {
+    oPacket.getDoc().replaceChild(oPacket._importNode(node, true),
+                                  oPacket.getNode());
+  }
 
-  return aNode;
+  return oPacket;
 };
 
